RStudio
=============================================================================

RStudio is an integrated development environment (IDE) for the 
[R programming language](http://www.r-project.org). Some of its
features include:

- Customizable workbench with all of the tools required to work with R in one
  place (console, source, plots, workspace, help, history, etc.).
- Syntax highlighting editor with code completion.
- Execute code directly from the source editor (line, selection, or file).
- Full support for authoring Sweave and TeX documents.
- Runs on Windows, Mac, and Linux, and has a community-maintained 
  [FreeBSD port](https://www.freshports.org/devel/RStudio/).
- Can also be run as a server, enabling multiple users to access the RStudio
  IDE using a web browser.

For more information on RStudio please visit the 
[project website](http://www.posit.co/).

Getting the Code
-----------------------------------------------------------------------------

RStudio is licensed under the AGPLv3, the terms of which are included in
the file COPYING. You can find our source code repository on GitHub at [https://github.com/rstudio/rstudio](https://github.com/rstudio/rstudio).

Documentation
-----------------------------------------------------------------------------

For information on how to use RStudio check out our
[online documentation](http://www.rstudio.com/ide/docs/). 

For documentation on running your own RStudio Server see the 
[server getting started](http://www.rstudio.com/ide/docs/server/getting_started)
guide.

See also the following files included with the distribution:

- COPYING - RStudio license (AGPLv3)
- NOTICE  - Additional open source software included with RStudio
- SOURCE  - How to obtain the source code for RStudio
- INSTALL - How to build and install RStudio from source
- VSCODE.md - How to get started with development using Visual Studio Code

If you have problems or want to share feedback with us please visit our
[community forum](https://community.rstudio.com/c/rstudio-ide). For other
inquiries you can also email us at [info@posit.co](mailto:info@posit.co). 
