# Utility Scripts

This folder contains scripts intended for use by engineers working on the RStudio IDE.

Use these with caution, and read the scripts before using them; they can definitely cause data loss.
