
Component Version Requirements
=============================================================================

RStudio requires relatively recent versions of a number of components. It is
therefore likely to only correctly configure and build on more recent Linux
systems. Specific version requirements for various components include:

- R 3.6.0
- CMake 3.19.8 or newer
- Boost 1.90.0
- patchelf 0.9 [Required only for Desktop]

Installing Dependencies for Debian/Ubuntu, Fedora, RedHat/CentOS, or OpenSUSE
=============================================================================

1) R installation - building RStudio requires that a version of R be
previously installed within a location where the R executable can be run from
the system shell (e.g. /usr/bin or /usr/local/bin). Building R from source
and then executing 'make install' will conform to this requirement as will
installing it using the system standard package management tools (apt-get,
yum, etc).

2) Run the install-dependencies script appropriate to your platform's
package management system

   ./install-dependencies-debian
   ./install-dependencies-yum
   ./install-dependencies-zypper

Note that this script will install Boost in the /opt/rstudio-tools/boost
directory (the /opt/rstudio-tools directory is used to insure that this
installation of boost doesn't conflict with other versions of boost installed
on the system). This part of the installation will take a few minutes since
it requires that boost be built from source for your system.


Installing Dependencies for Other Systems
=============================================================================

If the system you are installing on is either older than the minimum required
versions listed above or another Linux distribution entirely, then you should
do the following to make sure you have fulfilled all dependencies:

1) The Dockerfiles in the docker/jenkins folder contain the definitive
dependency requirements for the platforms that RStudio is built on and can
serve as a guide in determining which referenced libraries must be available
on your system.

2) Run the install-common script from dependencies/common (which will
install Google Web Toolkit, Hunspell dictionaries, MathJax, Boost 1.78, and
other dependencies).

3) If your system doesn't meet the minimum version requirements for CMake
you can download a Linux binary of the latest version of CMake here:

  - https://cmake.org/download/
