# README - `docs/reference`

Reference material for use by developers and agents working on the RStudio code.

Caution: we will try to keep it current and accurate, but always check the actual
code.

## ansi.md

[ansi.md](./ansi.md) - documentation on ANSI control code support in RStudio Console
