These are themes mostly copied from <https://github.com/mbadolato/iTerm2-Color-Schemes> with names to match the Ace theme names

They are used to generate the css mappings needed for the first 16 colors and
are parsed by `generate_xterm_16color_map()` included in `../compile-themes.R`

`download_theme_xrdb.R` is used to download the xrdb files, the URLs are listed in `theme_urls.tsv`
