# Adding data source icons

### File name matching

Data source icons are displayed in the connection dialog when the icon file name matches one or more of the references below. Name matching is done with case insensitive exact matching (minus the file extension). For example, references to `SQLServer` will match `sqlserver.png`. However, references to `microsoftsqlserver` will *not* match `sqlserver.png`.

There are five ways to match data source icons. For more info, see [Troubleshooting Connections](https://support.posit.co/hc/en-us/articles/115011264307).

1.  The driver name in `odbcinst.ini`

2.  The DSN name in `odbc.ini`

3.  The DSN name in `.odbc.ini`

4.  The snippet name on the system under `/etc/rstudio/connections/<snippet name>.R`

5.  The snippet name in an R package under `inst/rstudio/connections/<snippet name>.R` provided the R package has a valid [connections contract](https://rstudio.github.io/rstudio-extensions/rstudio-connections.html)

### Icon size and resolution

Each data source should have a large and small `png` icon.

#### Small ![](sqlite.png)

-   Name: `<data source>.png`
-   Size: 32px x 32px
-   Resolution: 72 px/in

#### Large ![](sqlitelarge.png)

-   Name: `<data source>large.png`
-   Size: 64px x 64px
-   Resolution: 72 px/in
