
#
# Execute the function definition below with Cmd + Enter.
# Then, run foo() to run the code.
# The debugger should open into a view of the document.
# Try editing the function definition while stepping through.
# The debug highlight should be removed, and a warning should be displayed.
#

foo <- function() {
   1 + 1
   browser()
   2 + 2
   3 + 3
   4 + 4
}
