#
# AppCommands.R
#
# Copyright (C) 2026 by Posit Software, PBC
#
# Unless you have received this program directly from Posit Software pursuant
# to the terms of a commercial license agreement with Posit Software, then
# this program is licensed to you under the terms of version 3 of the
# GNU Affero General Public License. This program is distributed WITHOUT
# ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF NON-INFRINGEMENT,
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. Please refer to the
# AGPL (http://www.gnu.org/licenses/agpl-3.0.txt) for more details.
#

#
# This file was automatically generated -- please do not modify it by hand.
# Generator: src/cpp/generate-commands-accessor.R
#

.rs.setVar("appCommands", list(
   setWorkingDir = "setWorkingDir",
   switchFocusSourceConsole = "switchFocusSourceConsole",
   reformatCode = "reformatCode",
   reformatDocument = "reformatDocument",
   newSourceDoc = "newSourceDoc",
   newRNotebook = "newRNotebook",
   newTextDoc = "newTextDoc",
   newCDoc = "newCDoc",
   newCppDoc = "newCppDoc",
   newHeaderDoc = "newHeaderDoc",
   newMarkdownDoc = "newMarkdownDoc",
   newPythonDoc = "newPythonDoc",
   newShellDoc = "newShellDoc",
   newStanDoc = "newStanDoc",
   newHtmlDoc = "newHtmlDoc",
   newJavaScriptDoc = "newJavaScriptDoc",
   newCssDoc = "newCssDoc",
   newD3Doc = "newD3Doc",
   newSweaveDoc = "newSweaveDoc",
   newRMarkdownDoc = "newRMarkdownDoc",
   newQuartoDoc = "newQuartoDoc",
   newQuartoPres = "newQuartoPres",
   newRShinyApp = "newRShinyApp",
   newRHTMLDoc = "newRHTMLDoc",
   newRDocumentationDoc = "newRDocumentationDoc",
   newRPresentationDoc = "newRPresentationDoc",
   newSqlDoc = "newSqlDoc",
   newRPlumberDoc = "newRPlumberDoc",
   openSourceDoc = "openSourceDoc",
   openSourceDocNewColumn = "openSourceDocNewColumn",
   reopenSourceDocWithEncoding = "reopenSourceDocWithEncoding",
   saveSourceDoc = "saveSourceDoc",
   saveSourceDocAs = "saveSourceDocAs",
   saveSourceDocWithEncoding = "saveSourceDocWithEncoding",
   saveAllSourceDocs = "saveAllSourceDocs",
   renameSourceDoc = "renameSourceDoc",
   copySourceDocPath = "copySourceDocPath",
   closeSourceDoc = "closeSourceDoc",
   closeOtherSourceDocs = "closeOtherSourceDocs",
   closeAllSourceDocs = "closeAllSourceDocs",
   executeAllCode = "executeAllCode",
   sourceFile = "sourceFile",
   previewJS = "previewJS",
   previewSql = "previewSql",
   sourceActiveDocument = "sourceActiveDocument",
   sourceActiveDocumentWithEcho = "sourceActiveDocumentWithEcho",
   runDocumentFromServerDotR = "runDocumentFromServerDotR",
   executeCode = "executeCode",
   executeCodeWithoutMovingCursor = "executeCodeWithoutMovingCursor",
   executeCodeWithoutFocus = "executeCodeWithoutFocus",
   executeToCurrentLine = "executeToCurrentLine",
   executeFromCurrentLine = "executeFromCurrentLine",
   executeCurrentFunction = "executeCurrentFunction",
   executeCurrentSection = "executeCurrentSection",
   executeLastCode = "executeLastCode",
   executeCurrentLine = "executeCurrentLine",
   executeCurrentStatement = "executeCurrentStatement",
   executeCurrentParagraph = "executeCurrentParagraph",
   insertChunk = "insertChunk",
   insertChunkR = "insertChunkR",
   insertChunkBash = "insertChunkBash",
   insertChunkD3 = "insertChunkD3",
   insertChunkGraphViz = "insertChunkGraphViz",
   insertChunkJulia = "insertChunkJulia",
   insertChunkMermaid = "insertChunkMermaid",
   insertChunkPython = "insertChunkPython",
   insertChunkRCPP = "insertChunkRCPP",
   insertChunkStan = "insertChunkStan",
   insertChunkSQL = "insertChunkSQL",
   switchToChunkR = "switchToChunkR",
   switchToChunkBash = "switchToChunkBash",
   switchToChunkPython = "switchToChunkPython",
   switchToChunkRCPP = "switchToChunkRCPP",
   switchToChunkStan = "switchToChunkStan",
   switchToChunkSQL = "switchToChunkSQL",
   insertSection = "insertSection",
   executePreviousChunks = "executePreviousChunks",
   executeSubsequentChunks = "executeSubsequentChunks",
   executeCurrentChunk = "executeCurrentChunk",
   executeNextChunk = "executeNextChunk",
   executeSetupChunk = "executeSetupChunk",
   goToHelp = "goToHelp",
   goToDefinition = "goToDefinition",
   sourceNavigateBack = "sourceNavigateBack",
   sourceNavigateForward = "sourceNavigateForward",
   markdownHelp = "markdownHelp",
   openRStudioIDECheatSheet = "openRStudioIDECheatSheet",
   openDataVisualizationCheatSheet = "openDataVisualizationCheatSheet",
   openDataImportCheatSheet = "openDataImportCheatSheet",
   openPackageDevelopmentCheatSheet = "openPackageDevelopmentCheatSheet",
   openDataWranglingCheatSheet = "openDataWranglingCheatSheet",
   openDataTransformationCheatSheet = "openDataTransformationCheatSheet",
   openRMarkdownCheatSheet = "openRMarkdownCheatSheet",
   openRMarkdownReferenceGuide = "openRMarkdownReferenceGuide",
   openShinyCheatSheet = "openShinyCheatSheet",
   openRoxygenQuickReference = "openRoxygenQuickReference",
   openSparklyrCheatSheet = "openSparklyrCheatSheet",
   openPurrrCheatSheet = "openPurrrCheatSheet",
   browseCheatSheets = "browseCheatSheets",
   toggleRmdVisualMode = "toggleRmdVisualMode",
   enableProsemirrorDevTools = "enableProsemirrorDevTools",
   quartoRenderDocument = "quartoRenderDocument",
   knitDocument = "knitDocument",
   previewHTML = "previewHTML",
   publishHTML = "publishHTML",
   compilePDF = "compilePDF",
   compileNotebook = "compileNotebook",
   synctexSearch = "synctexSearch",
   activateSource = "activateSource",
   layoutZoomSource = "layoutZoomSource",
   printSourceDoc = "printSourceDoc",
   vcsFileLog = "vcsFileLog",
   vcsFileDiff = "vcsFileDiff",
   vcsFileRevert = "vcsFileRevert",
   popoutDoc = "popoutDoc",
   returnDocToMain = "returnDocToMain",
   quickAddNext = "quickAddNext",
   findReplace = "findReplace",
   findNext = "findNext",
   findPrevious = "findPrevious",
   findSelectAll = "findSelectAll",
   findFromSelection = "findFromSelection",
   findAll = "findAll",
   replaceAndFind = "replaceAndFind",
   findInFiles = "findInFiles",
   refreshFindInFiles = "refreshFindInFiles",
   newSourceColumn = "newSourceColumn",
   fold = "fold",
   unfold = "unfold",
   foldAll = "foldAll",
   unfoldAll = "unfoldAll",
   jumpToMatching = "jumpToMatching",
   selectToMatching = "selectToMatching",
   expandToMatching = "expandToMatching",
   addCursorAbove = "addCursorAbove",
   addCursorBelow = "addCursorBelow",
   moveLinesUp = "moveLinesUp",
   moveLinesDown = "moveLinesDown",
   expandToLine = "expandToLine",
   copyLinesDown = "copyLinesDown",
   joinLines = "joinLines",
   removeLine = "removeLine",
   splitIntoLines = "splitIntoLines",
   editLinesFromStart = "editLinesFromStart",
   toggleDocumentOutline = "toggleDocumentOutline",
   expandSelection = "expandSelection",
   shrinkSelection = "shrinkSelection",
   goToNextSection = "goToNextSection",
   goToPrevSection = "goToPrevSection",
   goToNextChunk = "goToNextChunk",
   goToPrevChunk = "goToPrevChunk",
   goToStartOfCurrentScope = "goToStartOfCurrentScope",
   goToEndOfCurrentScope = "goToEndOfCurrentScope",
   expandRaggedSelection = "expandRaggedSelection",
   extractFunction = "extractFunction",
   extractLocalVariable = "extractLocalVariable",
   commentUncomment = "commentUncomment",
   blockIndent = "blockIndent",
   blockOutdent = "blockOutdent",
   reindent = "reindent",
   reflowComment = "reflowComment",
   setWorkingDirToActiveDoc = "setWorkingDirToActiveDoc",
   codeCompletion = "codeCompletion",
   findUsages = "findUsages",
   editRmdFormatOptions = "editRmdFormatOptions",
   knitWithParameters = "knitWithParameters",
   clearKnitrCache = "clearKnitrCache",
   clearPrerenderedOutput = "clearPrerenderedOutput",
   notebookExpandAllOutput = "notebookExpandAllOutput",
   notebookCollapseAllOutput = "notebookCollapseAllOutput",
   notebookClearOutput = "notebookClearOutput",
   notebookClearAllOutput = "notebookClearAllOutput",
   notebookToggleExpansion = "notebookToggleExpansion",
   renameInScope = "renameInScope",
   insertRoxygenSkeleton = "insertRoxygenSkeleton",
   insertSnippet = "insertSnippet",
   yankRegion = "yankRegion",
   yankBeforeCursor = "yankBeforeCursor",
   yankAfterCursor = "yankAfterCursor",
   pasteLastYank = "pasteLastYank",
   insertAssignmentOperator = "insertAssignmentOperator",
   insertPipeOperator = "insertPipeOperator",
   openNextFileOnFilesystem = "openNextFileOnFilesystem",
   openPreviousFileOnFilesystem = "openPreviousFileOnFilesystem",
   toggleSoftWrapMode = "toggleSoftWrapMode",
   toggleRainbowParens = "toggleRainbowParens",
   toggleRainbowFencedDivs = "toggleRainbowFencedDivs",
   newProject = "newProject",
   newProjectFromTemplate = "newProjectFromTemplate",
   openProject = "openProject",
   openProjectInNewWindow = "openProjectInNewWindow",
   shareProject = "shareProject",
   openSharedProject = "openSharedProject",
   projectMru0 = "projectMru0",
   projectMru1 = "projectMru1",
   projectMru2 = "projectMru2",
   projectMru3 = "projectMru3",
   projectMru4 = "projectMru4",
   projectMru5 = "projectMru5",
   projectMru6 = "projectMru6",
   projectMru7 = "projectMru7",
   projectMru8 = "projectMru8",
   projectMru9 = "projectMru9",
   projectMru10 = "projectMru10",
   projectMru11 = "projectMru11",
   projectMru12 = "projectMru12",
   projectMru13 = "projectMru13",
   projectMru14 = "projectMru14",
   clearRecentProjects = "clearRecentProjects",
   closeProject = "closeProject",
   projectOptions = "projectOptions",
   projectSweaveOptions = "projectSweaveOptions",
   setWorkingDirToProjectDir = "setWorkingDirToProjectDir",
   consoleClear = "consoleClear",
   consoleFind = "consoleFind",
   interruptR = "interruptR",
   restartR = "restartR",
   restartRClearOutput = "restartRClearOutput",
   restartRRunAllChunks = "restartRRunAllChunks",
   terminateR = "terminateR",
   activateConsole = "activateConsole",
   activateConsolePane = "activateConsolePane",
   layoutZoomConsole = "layoutZoomConsole",
   layoutZoomConsolePane = "layoutZoomConsolePane",
   activateConsolePanePane = "activateConsolePanePane",
   consoleActivateR = "consoleActivateR",
   consoleActivatePython = "consoleActivatePython",
   newFolder = "newFolder",
   touchSourceDoc = "touchSourceDoc",
   touchTextDoc = "touchTextDoc",
   touchCDoc = "touchCDoc",
   touchCppDoc = "touchCppDoc",
   touchHeaderDoc = "touchHeaderDoc",
   touchMarkdownDoc = "touchMarkdownDoc",
   touchPythonDoc = "touchPythonDoc",
   touchShellDoc = "touchShellDoc",
   touchStanDoc = "touchStanDoc",
   touchHtmlDoc = "touchHtmlDoc",
   touchJavaScriptDoc = "touchJavaScriptDoc",
   touchCssDoc = "touchCssDoc",
   touchD3Doc = "touchD3Doc",
   touchSweaveDoc = "touchSweaveDoc",
   touchRMarkdownDoc = "touchRMarkdownDoc",
   touchQuartoDoc = "touchQuartoDoc",
   touchRShinyApp = "touchRShinyApp",
   touchRHTMLDoc = "touchRHTMLDoc",
   touchSqlDoc = "touchSqlDoc",
   uploadFile = "uploadFile",
   copyFile = "copyFile",
   copyFileTo = "copyFileTo",
   moveFiles = "moveFiles",
   exportFiles = "exportFiles",
   renameFile = "renameFile",
   deleteFiles = "deleteFiles",
   refreshFiles = "refreshFiles",
   activateFiles = "activateFiles",
   layoutZoomFiles = "layoutZoomFiles",
   goToWorkingDir = "goToWorkingDir",
   setAsWorkingDir = "setAsWorkingDir",
   copyFilesPaneCurrentDirectory = "copyFilesPaneCurrentDirectory",
   openFilesInSinglePane = "openFilesInSinglePane",
   openEachFileInColumns = "openEachFileInColumns",
   setWorkingDirToFilesPane = "setWorkingDirToFilesPane",
   showFolder = "showFolder",
   vcsDiff = "vcsDiff",
   vcsCommit = "vcsCommit",
   vcsRevert = "vcsRevert",
   vcsViewOnGitHub = "vcsViewOnGitHub",
   vcsBlameOnGitHub = "vcsBlameOnGitHub",
   vcsShowHistory = "vcsShowHistory",
   vcsRefresh = "vcsRefresh",
   vcsRefreshNoError = "vcsRefreshNoError",
   vcsOpen = "vcsOpen",
   vcsIgnore = "vcsIgnore",
   vcsPull = "vcsPull",
   vcsPullRebase = "vcsPullRebase",
   vcsPush = "vcsPush",
   vcsCleanup = "vcsCleanup",
   vcsAddFiles = "vcsAddFiles",
   vcsRemoveFiles = "vcsRemoveFiles",
   activateVcs = "activateVcs",
   layoutZoomVcs = "layoutZoomVcs",
   vcsResolve = "vcsResolve",
   showPdfExternal = "showPdfExternal",
   openHtmlExternal = "openHtmlExternal",
   saveHtmlPreviewAsLocalFile = "saveHtmlPreviewAsLocalFile",
   saveHtmlPreviewAs = "saveHtmlPreviewAs",
   refreshHtmlPreview = "refreshHtmlPreview",
   showHtmlPreviewLog = "showHtmlPreviewLog",
   refreshPresentation = "refreshPresentation",
   presentationFullscreen = "presentationFullscreen",
   presentationHome = "presentationHome",
   presentationNext = "presentationNext",
   presentationPrev = "presentationPrev",
   presentationEdit = "presentationEdit",
   presentationViewInBrowser = "presentationViewInBrowser",
   presentationSaveAsStandalone = "presentationSaveAsStandalone",
   clearPresentationCache = "clearPresentationCache",
   activatePresentation2 = "activatePresentation2",
   layoutZoomPresentation2 = "layoutZoomPresentation2",
   presentation2Home = "presentation2Home",
   presentation2Next = "presentation2Next",
   presentation2Prev = "presentation2Prev",
   presentation2Edit = "presentation2Edit",
   presentation2Present = "presentation2Present",
   presentation2PresentFromBeginning = "presentation2PresentFromBeginning",
   presentation2Print = "presentation2Print",
   refreshPresentation2 = "refreshPresentation2",
   showToolbar = "showToolbar",
   hideToolbar = "hideToolbar",
   toggleToolbar = "toggleToolbar",
   toggleSidebar = "toggleSidebar",
   toggleSidebarLocation = "toggleSidebarLocation",
   zoomActualSize = "zoomActualSize",
   zoomIn = "zoomIn",
   zoomOut = "zoomOut",
   jumpTo = "jumpTo",
   goToFileFunction = "goToFileFunction",
   switchToTab = "switchToTab",
   previousTab = "previousTab",
   nextTab = "nextTab",
   firstTab = "firstTab",
   lastTab = "lastTab",
   goToLine = "goToLine",
   toggleFullScreen = "toggleFullScreen",
   moveTabLeft = "moveTabLeft",
   moveTabRight = "moveTabRight",
   moveTabToFirst = "moveTabToFirst",
   moveTabToLast = "moveTabToLast",
   activatePresentation = "activatePresentation",
   activateCompilePDF = "activateCompilePDF",
   activateFindInFiles = "activateFindInFiles",
   activateSourceCpp = "activateSourceCpp",
   activateRMarkdown = "activateRMarkdown",
   activateDeployContent = "activateDeployContent",
   activateMarkers = "activateMarkers",
   activateSQLResults = "activateSQLResults",
   activateDatabricks = "activateDatabricks",
   historySendToSource = "historySendToSource",
   historySendToConsole = "historySendToConsole",
   searchHistory = "searchHistory",
   loadHistory = "loadHistory",
   saveHistory = "saveHistory",
   historyRemoveEntries = "historyRemoveEntries",
   clearHistory = "clearHistory",
   historyDismissResults = "historyDismissResults",
   historyShowContext = "historyShowContext",
   historyDismissContext = "historyDismissContext",
   activateHistory = "activateHistory",
   layoutZoomHistory = "layoutZoomHistory",
   activateConnections = "activateConnections",
   layoutZoomConnections = "layoutZoomConnections",
   clearWorkspace = "clearWorkspace",
   refreshWorkspace = "refreshWorkspace",
   saveWorkspace = "saveWorkspace",
   loadWorkspace = "loadWorkspace",
   importDatasetFromFile = "importDatasetFromFile",
   importDatasetFromURL = "importDatasetFromURL",
   importDatasetFromCsv = "importDatasetFromCsv",
   importDatasetFromCsvUsingReadr = "importDatasetFromCsvUsingReadr",
   importDatasetFromCsvUsingBase = "importDatasetFromCsvUsingBase",
   importDatasetFromSAV = "importDatasetFromSAV",
   importDatasetFromSAS = "importDatasetFromSAS",
   importDatasetFromStata = "importDatasetFromStata",
   importDatasetFromXLS = "importDatasetFromXLS",
   activateEnvironment = "activateEnvironment",
   layoutZoomEnvironment = "layoutZoomEnvironment",
   refreshEnvironment = "refreshEnvironment",
   freeUnusedMemory = "freeUnusedMemory",
   toggleShowMemoryUsage = "toggleShowMemoryUsage",
   showMemoryUsageReport = "showMemoryUsageReport",
   nextPlot = "nextPlot",
   previousPlot = "previousPlot",
   savePlotAsImage = "savePlotAsImage",
   savePlotAsPdf = "savePlotAsPdf",
   copyPlotToClipboard = "copyPlotToClipboard",
   zoomPlot = "zoomPlot",
   removePlot = "removePlot",
   clearPlots = "clearPlots",
   refreshPlot = "refreshPlot",
   activatePlots = "activatePlots",
   layoutZoomPlots = "layoutZoomPlots",
   showManipulator = "showManipulator",
   installPackage = "installPackage",
   updatePackages = "updatePackages",
   refreshPackages = "refreshPackages",
   activatePackages = "activatePackages",
   layoutZoomPackages = "layoutZoomPackages",
   packratBootstrap = "packratBootstrap",
   packratOptions = "packratOptions",
   packratBundle = "packratBundle",
   packratHelp = "packratHelp",
   packratClean = "packratClean",
   packratCheckStatus = "packratCheckStatus",
   renvHelp = "renvHelp",
   renvSnapshot = "renvSnapshot",
   renvRestore = "renvRestore",
   versionControlHelp = "versionControlHelp",
   versionControlShowRsaKey = "versionControlShowRsaKey",
   versionControlProjectSetup = "versionControlProjectSetup",
   showProfiler = "showProfiler",
   startProfiler = "startProfiler",
   stopProfiler = "stopProfiler",
   profileCode = "profileCode",
   profileCodeWithoutFocus = "profileCodeWithoutFocus",
   saveProfileAs = "saveProfileAs",
   openProfile = "openProfile",
   profileHelp = "profileHelp",
   gotoProfileSource = "gotoProfileSource",
   openProfileInBrowser = "openProfileInBrowser",
   macPreferences = "macPreferences",
   showOptions = "showOptions",
   showCodeOptions = "showCodeOptions",
   showConsoleOptions = "showConsoleOptions",
   showAppearanceOptions = "showAppearanceOptions",
   showPackagesOptions = "showPackagesOptions",
   showRMarkdownOptions = "showRMarkdownOptions",
   showSweaveOptions = "showSweaveOptions",
   showSpellingOptions = "showSpellingOptions",
   showVcsOptions = "showVcsOptions",
   showPublishingOptions = "showPublishingOptions",
   showPythonOptions = "showPythonOptions",
   modifyKeyboardShortcuts = "modifyKeyboardShortcuts",
   editCodeSnippets = "editCodeSnippets",
   showCommandPalette = "showCommandPalette",
   clearCommandPaletteMru = "clearCommandPaletteMru",
   newTerminal = "newTerminal",
   activateTerminal = "activateTerminal",
   renameTerminal = "renameTerminal",
   closeTerminal = "closeTerminal",
   closeAllTerminals = "closeAllTerminals",
   clearTerminalScrollbackBuffer = "clearTerminalScrollbackBuffer",
   previousTerminal = "previousTerminal",
   nextTerminal = "nextTerminal",
   showTerminalInfo = "showTerminalInfo",
   interruptTerminal = "interruptTerminal",
   sendTerminalToEditor = "sendTerminalToEditor",
   sendToTerminal = "sendToTerminal",
   showTerminalOptions = "showTerminalOptions",
   openNewTerminalAtEditorLocation = "openNewTerminalAtEditorLocation",
   openNewTerminalAtFilePaneLocation = "openNewTerminalAtFilePaneLocation",
   sendFilenameToTerminal = "sendFilenameToTerminal",
   setTerminalToCurrentDirectory = "setTerminalToCurrentDirectory",
   helpBack = "helpBack",
   helpForward = "helpForward",
   helpHome = "helpHome",
   helpSearch = "helpSearch",
   printHelp = "printHelp",
   clearHelpHistory = "clearHelpHistory",
   helpPopout = "helpPopout",
   refreshHelp = "refreshHelp",
   raiseException = "raiseException",
   raiseException2 = "raiseException2",
   activateHelp = "activateHelp",
   layoutZoomHelp = "layoutZoomHelp",
   showAboutDialog = "showAboutDialog",
   showSplashScreen = "showSplashScreen",
   copyRStudioVersion = "copyRStudioVersion",
   showLicenseDialog = "showLicenseDialog",
   checkForUpdates = "checkForUpdates",
   helpUsingRStudio = "helpUsingRStudio",
   helpKeyboardShortcuts = "helpKeyboardShortcuts",
   showRequestLog = "showRequestLog",
   logFocusedElement = "logFocusedElement",
   debugDumpContents = "debugDumpContents",
   debugImportDump = "debugImportDump",
   refreshSuperDevMode = "refreshSuperDevMode",
   viewShortcuts = "viewShortcuts",
   editUserPrefs = "editUserPrefs",
   viewAllPrefs = "viewAllPrefs",
   clearUserPrefs = "clearUserPrefs",
   activateTutorial = "activateTutorial",
   layoutZoomTutorial = "layoutZoomTutorial",
   tutorialPopout = "tutorialPopout",
   tutorialBack = "tutorialBack",
   tutorialForward = "tutorialForward",
   tutorialZoom = "tutorialZoom",
   tutorialRefresh = "tutorialRefresh",
   tutorialStop = "tutorialStop",
   tutorialHome = "tutorialHome",
   activateChat = "activateChat",
   layoutZoomChat = "layoutZoomChat",
   activateViewer = "activateViewer",
   layoutZoomViewer = "layoutZoomViewer",
   viewerPopout = "viewerPopout",
   viewerBack = "viewerBack",
   viewerForward = "viewerForward",
   viewerZoom = "viewerZoom",
   viewerRefresh = "viewerRefresh",
   viewerSaveAllAndRefresh = "viewerSaveAllAndRefresh",
   viewerStop = "viewerStop",
   viewerClear = "viewerClear",
   viewerClearAll = "viewerClearAll",
   viewerSaveAsImage = "viewerSaveAsImage",
   viewerSaveAsWebPage = "viewerSaveAsWebPage",
   viewerCopyToClipboard = "viewerCopyToClipboard",
   viewerEditSource = "viewerEditSource",
   newSession = "newSession",
   suspendSession = "suspendSession",
   quitSession = "quitSession",
   forceQuitSession = "forceQuitSession",
   updateCredentials = "updateCredentials",
   diagnosticsReport = "diagnosticsReport",
   openDeveloperConsole = "openDeveloperConsole",
   reloadUi = "reloadUi",
   showGpuDiagnostics = "showGpuDiagnostics",
   showA11yDiagnostics = "showA11yDiagnostics",
   showLogFiles = "showLogFiles",
   rstudioSupport = "rstudioSupport",
   rstudioCommunityForum = "rstudioCommunityForum",
   rstudioAgreement = "rstudioAgreement",
   showWarningBar = "showWarningBar",
   signOut = "signOut",
   loadServerHome = "loadServerHome",
   clearBuild = "clearBuild",
   findBuild = "findBuild",
   buildAll = "buildAll",
   buildIncremental = "buildIncremental",
   buildFull = "buildFull",
   devtoolsLoadAll = "devtoolsLoadAll",
   serveQuartoSite = "serveQuartoSite",
   cleanAll = "cleanAll",
   buildSourcePackage = "buildSourcePackage",
   buildBinaryPackage = "buildBinaryPackage",
   roxygenizePackage = "roxygenizePackage",
   checkPackage = "checkPackage",
   testPackage = "testPackage",
   testTestthatFile = "testTestthatFile",
   testShinytestFile = "testShinytestFile",
   stopBuild = "stopBuild",
   buildToolsProjectSetup = "buildToolsProjectSetup",
   activateBuild = "activateBuild",
   layoutZoomBuild = "layoutZoomBuild",
   newConnection = "newConnection",
   removeConnection = "removeConnection",
   disconnectConnection = "disconnectConnection",
   refreshConnection = "refreshConnection",
   undoDummy = "undoDummy",
   redoDummy = "redoDummy",
   cutDummy = "cutDummy",
   copyDummy = "copyDummy",
   pasteDummy = "pasteDummy",
   pasteWithIndentDummy = "pasteWithIndentDummy",
   selectAllDummy = "selectAllDummy",
   mru0 = "mru0",
   mru1 = "mru1",
   mru2 = "mru2",
   mru3 = "mru3",
   mru4 = "mru4",
   mru5 = "mru5",
   mru6 = "mru6",
   mru7 = "mru7",
   mru8 = "mru8",
   mru9 = "mru9",
   mru10 = "mru10",
   mru11 = "mru11",
   mru12 = "mru12",
   mru13 = "mru13",
   mru14 = "mru14",
   clearRecentFiles = "clearRecentFiles",
   debugBreakpoint = "debugBreakpoint",
   debugClearBreakpoints = "debugClearBreakpoints",
   debugContinue = "debugContinue",
   debugStop = "debugStop",
   debugStep = "debugStep",
   debugStepInto = "debugStepInto",
   debugFinish = "debugFinish",
   debugHelp = "debugHelp",
   errorsMessage = "errorsMessage",
   errorsTraceback = "errorsTraceback",
   errorsBreak = "errorsBreak",
   showDiagnosticsActiveDocument = "showDiagnosticsActiveDocument",
   showDiagnosticsProject = "showDiagnosticsProject",
   reloadShinyApp = "reloadShinyApp",
   shinyRunInPane = "shinyRunInPane",
   shinyRunInViewer = "shinyRunInViewer",
   shinyRunInBrowser = "shinyRunInBrowser",
   shinyRecordTest = "shinyRecordTest",
   shinyRunAllTests = "shinyRunAllTests",
   shinyCompareTest = "shinyCompareTest",
   reloadPlumberAPI = "reloadPlumberAPI",
   plumberRunInPane = "plumberRunInPane",
   plumberRunInViewer = "plumberRunInViewer",
   plumberRunInBrowser = "plumberRunInBrowser",
   rsconnectDeploy = "rsconnectDeploy",
   rsconnectConfigure = "rsconnectConfigure",
   rsconnectManageAccounts = "rsconnectManageAccounts",
   browseAddins = "browseAddins",
   startJob = "startJob",
   sourceAsJob = "sourceAsJob",
   clearBackgroundJobs = "clearBackgroundJobs",
   activateBackgroundJobs = "activateBackgroundJobs",
   runSelectionAsBackgroundJob = "runSelectionAsBackgroundJob",
   startWorkbenchJob = "startWorkbenchJob",
   sourceAsWorkbenchJob = "sourceAsWorkbenchJob",
   runSelectionAsWorkbenchJob = "runSelectionAsWorkbenchJob",
   activateWorkbenchJobs = "activateWorkbenchJobs",
   sortWorkbenchJobsRecorded = "sortWorkbenchJobsRecorded",
   sortWorkbenchJobsState = "sortWorkbenchJobsState",
   checkSpelling = "checkSpelling",
   wordCount = "wordCount",
   layoutZoomCurrentPane = "layoutZoomCurrentPane",
   layoutEndZoom = "layoutEndZoom",
   layoutConsoleOnLeft = "layoutConsoleOnLeft",
   layoutConsoleOnRight = "layoutConsoleOnRight",
   paneLayout = "paneLayout",
   restoreDefaultPaneAndTabLayout = "restoreDefaultPaneAndTabLayout",
   restoreDefaultPaneAndTabLayoutNoPrompt = "restoreDefaultPaneAndTabLayoutNoPrompt",
   maximizeConsole = "maximizeConsole",
   maximizeSource = "maximizeSource",
   maximizeTabSet1 = "maximizeTabSet1",
   maximizeTabSet2 = "maximizeTabSet2",
   toggleEditorTokenInfo = "toggleEditorTokenInfo",
   layoutZoomLeftColumn = "layoutZoomLeftColumn",
   layoutZoomRightColumn = "layoutZoomRightColumn",
   layoutZoomSidebar = "layoutZoomSidebar",
   focusLeftSeparator = "focusLeftSeparator",
   focusRightSeparator = "focusRightSeparator",
   focusCenterSeparator = "focusCenterSeparator",
   focusSourceColumnSeparator = "focusSourceColumnSeparator",
   focusSidebarSeparator = "focusSidebarSeparator",
   showFileMenu = "showFileMenu",
   showEditMenu = "showEditMenu",
   showCodeMenu = "showCodeMenu",
   showViewMenu = "showViewMenu",
   showPlotsMenu = "showPlotsMenu",
   showSessionMenu = "showSessionMenu",
   showBuildMenu = "showBuildMenu",
   showDebugMenu = "showDebugMenu",
   showProfileMenu = "showProfileMenu",
   showToolsMenu = "showToolsMenu",
   showHelpMenu = "showHelpMenu",
   toggleScreenReaderSupport = "toggleScreenReaderSupport",
   toggleTabKeyMovesFocus = "toggleTabKeyMovesFocus",
   showAccessibilityOptions = "showAccessibilityOptions",
   focusMainToolbar = "focusMainToolbar",
   speakEditorLocation = "speakEditorLocation",
   focusConsoleOutputEnd = "focusConsoleOutputEnd",
   focusNextPane = "focusNextPane",
   focusPreviousPane = "focusPreviousPane",
   showAccessibilityHelp = "showAccessibilityHelp",
   copilotDiagnostics = "copilotDiagnostics",
   copilotSignIn = "copilotSignIn",
   copilotSignOut = "copilotSignOut",
   copilotStatus = "copilotStatus",
   copilotRequestCompletions = "copilotRequestCompletions",
   copilotAcceptNextWord = "copilotAcceptNextWord",
   copilotToggleAutomaticCompletions = "copilotToggleAutomaticCompletions",
   layoutZoomDatabricks = "layoutZoomDatabricks",
   showDomElements = "showDomElements",
   showShortcutCommand = "showShortcutCommand",
   crashDesktopApplication = "crashDesktopApplication"
))
