---
name: Feature request
about: Request a new feature.
title: ''
labels: ['enhancement']
assignees: ''

---

<!--

Thanks for taking the time to file a feature request! Please take the time to search for an existing feature request, to avoid creating duplicate requests. If you find an existing feature request, please give it a thumbs-up reaction, as we'll use these reactions to help prioritize the implementation of these features in the future.

If the feature has not yet been filed, then please describe the feature you'd like to see become a part of RStudio. See:

https://github.com/rstudio/rstudio/wiki/Writing-Good-Feature-Requests

for a guide on how to write good feature requests.

-->

### Problem Statement

> Describe the problem for which you are creating this issue.

### Proposed Behaviour

> Describe the proposed behaviour which would solve the problem above.

### QA Notes

> Once you have opened a PR, add additional information for QA on how to validate the change, paying special attention to the level of risk, adjacent areas that could be affected by the change, and any important contextual information not present in the original description.

