---
title: "Posit Assistant Terms and Conditions"
date-meta: 2026-07-02
aliases:
  - posit-ai-terms.html
---

# Posit Assistant specific EULA

Posit Software, PBC

This End User License Agreement including all Order Forms, the Software License Descriptions and all documents linked to herein and therein (collectively, this "Agreement") is a legal agreement between you ("you") and Posit Software, PBC, a Delaware corporation with its principal place of business at 250 Northern Avenue, Suite 410, Boston, Massachusetts 02210 ("Posit") (formerly known as RStudio, PBC), that governs your use of Posit’s proprietary Software, directly from Posit or indirectly through a Posit authorized reseller or distributor (a "Reseller"). It is effective between you and Posit as of the Subscription Start Date. Your execution of an Order Form constitutes a binding noncancelable commitment to license the Software set forth in the Order Form under the terms and conditions of this Agreement. All Order Forms, the Software License Descriptions and all documents linked to herein and therein are incorporated into this End User License Agreement by reference.

IMPORTANT -- PLEASE READ CAREFULLY BEFORE YOU DOWNLOAD OR USE THE SOFTWARE

By clicking on the “accept” button at the end of this document or by accessing, executing or otherwise using the Software, you acknowledge that you have read this Agreement, understand it and agree to be bound by its terms and conditions. If you are not willing to be bound by the terms of this Agreement, do not access or use the Software.

If you are using the Software in your capacity as employee or agent of a company or organization, then any references to “you” in this agreement shall refer to such entity and not to you in your personal capacity. You warrant that you are authorized to legally bind the company or organization on whose behalf you are accessing the Software. If you are not so authorized, then neither you nor your company or organization may use the Software in any manner whatsoever.

1\.  Definitions.

1.1 "Affiliate" means any entity which directly or indirectly controls, is controlled by, or is under common control with the subject entity. "Control," for purposes hereof, means (a) the legal power to direct or cause the direction of the general management and policies of an entity whether directly or indirectly and whether through the ownership of voting securities, by contract, or otherwise, or (b) the beneficial ownership, whether direct or indirect, of fifty percent (50%) or more of the voting securities or other ownership interest or other comparable equity interests of an entity.

1.2 "Customer Application" means a software application independently developed by you or your Users and/or deployed by you or your Users using the Software.

1.3 "Documentation" means the published and generally available instructions for Software installation, use, and administration set forth at docs.posit.co, as updated from time to time to reflect Updates, and the Software License Descriptions.

1.4 "Order Form" means Posit’s order form which is separately executed by you and Posit and which lists the Software ordered by you, the annual subscription fees payable for such Software, the Subscription Term, the Software License Descriptions, and any additional terms and conditions agreed to by the parties. All Order Forms, executed by the parties and their respective Affiliates, are incorporated herein by reference.

1.5 "Open Source Language(s)" means open source programming languages and software environments for statistical computing and graphics made available in source code form for free by third parties, such as "R" or Python.

1.6 "Posit Support Program" means Posit’s Support Program for the Software at https://posit.co/about/support-agreement/, as the same may be updated from time to time by Posit in its discretion; provided, that any updates will not take effect with respect to your subscription to the Software until renewal of the Subscription Term for such Software.

1.7 "Software License Descriptions" means the license entitlements applicable to the Software set forth at https://posit.co/about/software-license-descriptions/, as updated from time to time, provided, that for Software that is subject to a then-current Subscription Term, such updates will not take effect until the renewal of the Subscription Term for such Software.

1.8 "Software" means the object code version of the proprietary Posit software program(s) set forth on an Order Form (or downloaded by you as part of a free trial pursuant to Section 2 below) and all Updates provided to you by Posit. "Software" expressly excludes Posit's Posit Cloud and shinyapps.io online services for which the terms of service are as set forth at https://posit.co/about/posit-service-terms-of-use/ as the same are updated from time to time.

1.9 "Subscription Term" means the period of time set forth on the Order Form during which you are licensed to use the Software in accordance with this Agreement and the Order Form and which period of time commences on the Subscription Start Date.

1.10 "Subscription Start Date" means the date a license key to activate the Software is provided to you.

1.11 "Support Services" mean the technical support services for the Software made generally commercially available by Posit to its customers pursuant to and as described in the Posit Support Program.

1.12 "Update" means any subsequent release of the Software that Posit makes generally commercially available to its customers pursuant to the Support Program. Updates do not include any Software that is marketed and priced separately by Posit.

1.13 "User" means an individual who is authorized by you or your Affiliates to use the Software, has been given access credentials by you to access and use the Software, and who is bound by enforceable terms at least as protective of the Software and the Posit Confidential Information as this Agreement.

1.14 "Third-Party Commercial Platform" means any platform, environment, product, or service operated, hosted, distributed, or otherwise made available by a person or entity (other than you for your own internal business purposes) under which third parties are granted access to software, computing environments, or development tools, whether on a hosted, software-as-a-service, platform-as-a-service, managed, self-managed, or on-premise basis, in each case in exchange for payment, subscription, or other consideration paid to such person or entity. For clarity, a platform or environment that you operate solely for your own internal business purposes and solely for access by your Users is not a Third-Party Commercial Platform.

2\.  Free Trial. If you either register on the Posit web site for a free trial of the Software or otherwise order a free trial version of the Software, Posit will make the Software available to you free of charge (the "Trial Version") until the end of the free trial period for which you registered or ordered the applicable Software. You may only use the Trial Version to review, demonstrate, and evaluate the Software, and the Trial Version may have limited functionality. The Trial Version will cease operating after the applicable time period based on an internal metering mechanism within the Trial Version itself. Regardless of any such metering, you must stop use at the end of the free trial period. Additional trial terms and conditions may appear on the trial registration web page or Order Form. Any such additional terms and conditions are incorporated into this Agreement by reference and are legally binding. You shall not (A) install or use more than one copy of the Trial Version or use the Trial Version on more than one computer except, if necessary, for the sole purpose of evaluating load balancing features, (B) download the Trial Version under more than one username, (C) alter the contents of a hard drive or computer system to enable the use of the Trial Version for an aggregate period in excess of the trial period for one license to such Trial Version, (D) disclose the results of software performance benchmarks obtained using the Trial Version to any third party without Posit’s prior written consent, (E) use the Trial Version to update Software that is no longer eligible for Support Services, or (F) use the Trial Version for any purpose other than the sole purpose of determining whether to purchase a license to the Software. NOTWITHSTANDING SECTION 9 (LIMITED WARRANTY AND DISCLAIMER), DURING A FREE TRIAL, THE TRIAL VERSION AND ANY APPLICABLE SERVICES ARE PROVIDED "AS-IS" WITHOUT WARRANTY.

3\.  License Grant.

3.1 License Grant Subject to all terms and conditions set forth in this Agreement, the Software License Descriptions, and the applicable Order Form, Posit grants you a limited, worldwide, nonexclusive, royalty-free license (without right of sublicense) during the Subscription Term to install and use the Software, and to permit your Users to use the Software, for your own business purposes, in accordance with the Documentation and subject to the license limitations and entitlements set forth in the applicable Order Form and the Software License Descriptions. Notwithstanding anything to the contrary herein, you may make a reasonable number of copies of the Software for the sole purpose of backing-up and archiving the Software; provided, that any such copies may not be used in a live production environment unless you have ordered a subscription for a High Availability Server or an Execution Server (as such terms are defined in the Software License Descriptions) pursuant to an Order Form. Further, unless you have purchased a subscription to Staging Software (as such term is defined in the Software License Descriptions), you may not use copies of the Software for purposes of testing changes to your computing environment or testing Customer Applications, scripts, or documents before deploying them to your Users. All copies of the Software are subject to this Agreement and must contain the same titles, trademarks, and copyrights as the original. You are responsible for the compliance of your Users with the terms and conditions of this Agreement, including their compliance with the Documentation and for any breach of this Agreement by your Users. At Posit’s written request, and not more than once every twelve (12) months (or more frequently if a previous violation has taken place but only to confirm the violation has been corrected and not to exceed more than once per calendar quarter), you will certify to Posit that you and your Users are in compliance with this Agreement and the Documentation.

3.2 Delivery. Delivery of the Software shall be made by electronic means and such delivery shall be deemed to have been made as of the Subscription Start Date. Notwithstanding the foregoing, where the Order Form provides for physical delivery or where electronic delivery is not possible given the nature of the requirements, delivery shall be deemed to have been made upon the transfer of the Software to the shipping agent. Posit’s Software (other than Trial Versions) may transmit license-related data at the time of license activation to a license server in order to activate your license and provide you with notifications, including notifications of Updates, and to protect you and Posit against unlicensed or illegal use of the Software. Notwithstanding the foregoing, if you desire that license-related data not be transmitted to Posit, you may elect to use the off-line activation mechanism as provided in the Software, in which case license-related data will not be transmitted. You acknowledge that the Software may include licensing technology, measures to prevent activating the Software on more than one physical or virtual computer, and other security measures designed to limit usage of the Software to that which is licensed. Such measures will not interfere with your normal and permitted operation of the Software as licensed hereunder.

3.3 Virtualization Technology; Third Party Hosting Provider. The Software may be installed within a virtual (or otherwise emulated) hardware system so long as the use of the Software meets the terms of the Software License Descriptions and these virtual machines are run on hardware you own, lease or otherwise control (including for such purpose the hardware of a third party hosting provider that hosts the Software for your benefit, such as Amazon Web Services). Virtualization technology may not be used to circumvent licensing terms or restrictions. Notwithstanding anything to the contrary herein, Posit grants you the right to sublicense the right to install and use the Software to a third party hosting provider for the sole purpose of enabling the third party hosting provider to host the Software on your behalf and for your exclusive benefit. You agree that you will be liable to Posit for any breach of this Agreement or unauthorized use of the Software by such third party hosting provider.

3.4 Restrictions; Export Laws.

3.4.1 Restrictions. You shall not and shall not permit any third party to: (a) use the Software other than in accordance with the Documentation, (b) use the Software in violation of the license restrictions applicable to the Software, (c) copy the Software (except for a reasonable number of backup and archival copies to the extent and as permitted in Section 3.1 above), (d) modify, adapt, transfer, distribute, make publicly available, resell, rent, lease, sublicense or loan the Software or create or prepare derivative works based upon the Software or any part thereof, (e) use the Software in a service bureau or application service provider environment, in a commercial time share arrangement, or for purposes of providing services to a third party other than in connection with deploying your Customer Applications created using the Software as permitted in the last two sentences of this Section 3.4.1, (f) combine all or any part of the Software with other software or use all or any part of the Software to create a new product, application or service, provided, that for purposes of clarity, you and your Users may use the Software to develop and deploy Customer Applications to the extent enabled by the Software and so long as you comply with the terms and conditions of this Agreement and do not infringe on or violate Posit’s intellectual property rights in the Software; (g) develop, resell, distribute or make available any service, software or device incorporating all or any part of the Software, (h) use the Software except on your own internal computer networks on devices that you own, lease or otherwise control (including for such purpose the hardware of a third party hosting providers that hosts the Software for your benefit, such as Amazon Web Services), (i) use the Software in contravention of, or to violate, any applicable laws, rules, regulations or guidelines or to develop Customer Applications that contravene or violate any applicable laws, rules, regulations or guidelines, (j) remove or alter any trademark, trade names, product names, logo, copyright or other proprietary notices, legends, symbols or labels from the Software, (k) decompile, disassemble, reverse engineer or otherwise attempt to derive the source code of the Software or attempt to modify, alter, or circumvent the license control and protection mechanisms within the Software, or (l) deploy, install, host, integrate, embed, make available, or otherwise enable use of the Software, either directly or as integrated within or accessed through RStudio or any other host integrated development environment, within, as part of, or as a feature or service of any Third-Party Commercial Platform, or otherwise provide the Software (alone or in combination with RStudio or any other software) to any third party as, or as part of, a commercial offering or service. The license granted in Section 3.1 does not extend to use of the Software within or in connection with any Third-Party Commercial Platform, and no such use is permitted under this Agreement. For the avoidance of doubt, the foregoing restriction regarding Third-Party Commercial Platforms and uses of the Software in connection with commercial offering or service, applies regardless of whether the Software is provided on a hosted, software-as-a-service, platform-as-a-service, managed, self-managed, or on-premise basis. Use of the Software in connection with the provision of software, development environments, or computing services to third parties is available solely through Posit Workbench under a separate subscription and Order Form. If 3.4(k) is prohibited by applicable law, you shall provide Posit with a detailed prior written notice of any such intention to reverse engineer the Software and shall provide Posit with a right of first refusal to perform such work at rates equal to those proposed by a recognized third-party software services provider for such work. You may only use Disaster Recovery Software and Staging Software as described in the Software License Descriptions. To the extent users of your Customer Applications access and use your Customer Applications as enabled through use of the Software, such users shall be deemed "Users" for all purposes hereunder except that such Users shall be restricted to viewing your directory of Customer Applications and to using and viewing the output of your Customer Applications. Such Users may not be authorized to, and must not, develop, publish, or modify Customer Applications. You agree that you will require your Users to comply with all applicable laws, rules and regulations and this Agreement in their use of your Customer Applications will comply with. 

3.4.2 Export. You may not export the Software or Customer Applications into any country prohibited by the United States Export Administration Act and the regulations thereunder. You acknowledge that the release of software, technology, or data to a non-US person within the United States or elsewhere abroad is deemed to be an export to the non-U.S. person’s home country or countries. You acknowledge that the export of any Software or Customer Applications is subject to export controls and you agree that the Software and any Customer Applications, and the direct or indirect product thereof, will not be exported (or re-exported from a country of installation), directly or indirectly, unless you obtain all necessary licenses from the U.S. Department of Commerce or other applicable agency or governmental body as required under applicable law or you qualify for exemptions or exceptions. You shall comply with the export laws and regulations of the United States and other applicable jurisdictions in using the Software, including without limitation the International Traffic in Arms Regulations (ITAR), 22 CFR 120 – 130, the Export Administration Regulations (EAR), 15 CFR 730 – 799, and the Foreign Assets Control Regulations (FACR), 31 CFR 500 – 599. Without limiting the foregoing, (i) you represent that neither you nor your Users are not named on any U.S. government list of persons or entities prohibited from receiving exports, and (ii) you shall not use, and shall not permit your Users to access or use, the Software or Customer Applications in violation of any U.S. export embargo, prohibition or restriction. You acknowledge and agree that the Software is prohibited for export or re-export to Cuba, North Korea, Iran, Libya, Syria and Sudan or to any person or entity on the U.S. Department of Commerce Denied Persons List, Entity List, or Unverified List, U.S. State Department’s Nonproliferation Sanctions List, or on the U.S. Department of Treasury’s lists of Specially Designated Nationals, Specially Designated Narcotics Traffickers or Specially Designated Terrorists, as such is changed from time to time.

3.5 Third Party Materials. You acknowledge and agree that the Software is intended for use with Open Source Languages, and, as such, interoperates with certain open source components, including without limitation open source packages and libraries, as part of its basic functionality and that to use the Software, you will need to separately license Open Source Languages from the licensor. Posit is not responsible for Open Source Languages and does not assume any obligations or liability with respect to your or your Users’ use of Open Source Languages. Further, you may elect to use, or Posit may make available to you or your Users for download, access or use, certain third party components, packages, applications, services, data, content or resources (collectively all of the foregoing, "Third Party Materials"). Posit may make available Third Party Materials via Posit’s website or may provide links to third party websites where you may purchase and/or download or access Third Party Materials or the Software may enable you to download, or to access and use, such Third Party Materials. You acknowledge and agree that Third Party Materials may be protected by intellectual property rights which are owned by the third-party providers or their licensors and not Posit. Accordingly, you acknowledge and agree that your use of Third Party Materials may be subject to separate terms between you and the relevant third party and you acknowledge and that Posit is not responsible for Third Party Materials and makes no representations or warranties with respect thereto, all of which are provided "as is". Posit assumes no liability or other obligations with respect thereto and, without limiting the foregoing, is not liable for any loss or damage that you or your Users may experience as a result of the use or access thereof. Third Party Materials are the sole responsibility of the third party from which they originated.

3.6 Open Source Software. You acknowledge that certain third party software incorporated in the Software requires that Posit provide copyright notices and/or additional terms and conditions to you, which copyright notices and additional terms and conditions may be accessed by you at https://www.posit.co/about/opensourcelicenses/ and are incorporated by reference into this Agreement. Use, reproduction and distribution of those components of the Software that are licensed under an open source software license are governed solely by the terms of that open source software license and not this Agreement.

4\. Support Services. During a Subscription Term, for the fee, if any, set forth in the applicable Order Form, Posit will provide you (through your designated representatives) with Support Services, including Updates, in accordance with and subject to the terms of the Support Program. Posit may use technical information provided by you relating to the Software as part of the Support Services, including for product support and development. As part of the Support Services, Posit may make available bug lists, planned feature lists, and other supplemental materials, all of which are Confidential Information of Posit and for which Posit makes no representations or warranties of any kind.

5\. Fees and Payments. You shall pay Posit the fees, charges and other amounts specified in an Order Form within forty-five (45) days from your receipt of Posit’s complete, accurate and undisputed invoice. If your internal procedures require that an invoice be submitted against a purchase order before payment can be made, you shall be responsible for issuing such purchase order to Posit prior to the Software license key being delivered and your failure to do so will not extend or obviate your responsibilities to pay all fees, charges and other amounts in accordance with this Agreement and the Order Form. All fees, charges, and other amounts are payable in United States dollars and, except as expressly otherwise provided herein, are nonrefundable. You shall be responsible for taxes levied on any transaction under this Agreement, including all federal, state, and local taxes, levies, and assessments, excluding any taxes based on Posit’s net income, employees, or property.

6\. Confidentiality.

6.1 Confidential Information. During the term of this Agreement, each party will regard any information provided to it by the other party and designated in writing as proprietary or confidential to be confidential ("Confidential Information"). Confidential Information shall also include information which, to a reasonable person familiar with the disclosing party’s business and the industry in which it operates, is of a confidential or proprietary nature. A party will not disclose the other party’s Confidential Information to any third party without the prior written consent of the other party, nor make use of any of the other party’s Confidential Information except in its performance under this Agreement. Each party accepts responsibility for the actions of its agents or employees and shall protect the other party’s Confidential Information in the same manner as it protects its own valuable confidential information, but in no event shall less than reasonable care be used. For the avoidance of doubt, the parties expressly agree that the Software, the fees payable under an Order Form and the terms of this Agreement are the Confidential Information of Posit. A receiving party shall promptly notify the disclosing party upon becoming aware of a breach or threatened breach hereunder, and shall cooperate with any reasonable request of the disclosing party in enforcing its rights.

6.2 Exclusions. Information will not be deemed Confidential Information hereunder if such information: (i) is known prior to receipt from the disclosing party, without any obligation of confidentiality; (ii) becomes known to the receiving party directly or indirectly from a source other than one having an obligation of confidentiality to the disclosing party; (iii) becomes publicly known or otherwise publicly available, except through a breach of this Agreement; or (iv) is independently developed by the receiving party without use of or reference to the disclosing party’s Confidential Information or breach of this Agreement. The receiving party may disclose Confidential Information pursuant to the requirements of applicable law (including without limitation applicable state or federal regulations which may require you to make disclosure pursuant to and as limited by such regulations, such as freedom of information regulations), legal process or government regulation, provided that it gives the disclosing party reasonable prior written notice to permit the disclosing party to contest such disclosure, and such disclosure is otherwise limited to the required disclosure.

6.3 Specific Performance. If the receiving party discloses or uses (or threatens to disclose or use) any Confidential Information in breach of this Section 6, the disclosing party shall have the right, in addition to any other remedies available to it, to seek injunctive relief to enjoin such acts, it being specifically acknowledged by the parties that any other available remedies are inadequate.

7\. Ownership; Feedback.

7.1 Posit Software. Posit and its licensors retain ownership of all right, title, and interest in and to the Software, including all intellectual property rights therein and thereto, and Posit reserves all rights not specifically granted in this Agreement. If the Software was acquired outside the United States, then you agree and assent to the adherence to all applicable international treaties regarding intellectual property rights which shall also apply. You shall take all reasonable precautions to prevent unauthorized or improper use or disclosure of the Software. Posit and other trademarks contained in the Software are trademarks or registered trademarks of Posit Software, PBC in the United States and/or other countries.

7.2 Customer Applications. To the extent you use the Software to develop and deploy Customer Applications, you and your licensors retain ownership of all right, title, and interest in and to the Customer Applications. Posit shall have no right to access or use your Customer Applications unless you separately grant Posit a license or right to access and use your Customer Applications.

7.3 Feedback. You and you Users may, from time to time, voluntarily make known to Posit suggestions, enhancement requests, techniques, know-how, comments, feedback, or other input to Posit with respect to the Software (collectively, "Suggestions"). Unless otherwise agreed to in writing by the parties with respect to any Suggestion, Posit shall have a royalty-free, worldwide, irrevocable, perpetual license to use, disclose, reproduce, license, distribute, and exploit any Suggestion without restriction or obligation of any kind, on account of confidential information, intellectual property rights, or otherwise, and may incorporate into its services any service, product, technology, enhancement, documentation, or other development ("Improvement") incorporating or derived from any Suggestion with no obligation to license or to make available the Improvement to you or any other person or entity.

8\. Limited Warranty.

8.1 Software Warranty. Except with respect to the Trial Version of the Software for which Posit provides no representations, warranties, or covenants, Posit represents, warrants and covenants that (a) the Software will perform in conformity with its Documentation; (b) any Support Services will be provided in a professional manner, with reasonable skill and care and in conformance with generally accepted software industry standards; and (c) as of the Subscription Start Date, the Software has been scanned using the most recent version of a leading commercially available virus scanning program designed to detect and remediate known viruses and other harmful and malicious code. Your exclusive remedy for Posit’s breach of the foregoing warranties is that Posit will, at its option and at no cost to you, either (a) provide remedial services necessary to enable the Software or Support Services to conform to the warranty, or (b) replace any defective Software or media. If neither of the foregoing options is commercially feasible within a reasonable period of time, upon your return of the affected Software to Posit, Posit will refund all prepaid fees for the unused remainder of the applicable Subscription Term following the date of termination for the affected Software and this Agreement and any associated Order Forms for the affected Software will immediately terminate without further action of the parties. You agree to provide Posit with a reasonable opportunity to remedy any breach and reasonable assistance in remedying any nonconformities.

8.2 DISCLAIMER. EXCEPT AS SET FORTH IN THE FOREGOING LIMITED WARRANTY IN SECTION 8.1, THE SOFTWARE IS PROVIDED "AS IS" AND POSIT AND ITS LICENSORS DISCLAIM ALL OTHER WARRANTIES AND REPRESENTATIONS, WHETHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE, AND EXPRESSLY DISCLAIM THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT. POSIT DOES NOT REPRESENT OR WARRANT THAT THE SOFTWARE IS ERROR FREE OR THAT ALL ERRORS CAN BE CORRECTED. THE SOFTWARE IS NOT DESIGNED, INTENDED, OR LICENSED FOR USE IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE CONTROLS, INCLUDING WITHOUT LIMITATION, THE DESIGN, CONSTRUCTION, MAINTENANCE, OR OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, AND LIFE SUPPORT OR WEAPONS SYSTEMS. POSIT SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR SUCH PURPOSES. No oral or written information or advice given by Posit, its resellers, dealers, distributors, agents, representatives or employees shall create any warranty or in any way increase any warranty provided herein.

8.3 Limitation. If applicable law requires any warranties other than the foregoing, all such warranties are limited in duration to ninety (90) days from the date of delivery. Some jurisdictions do not allow the exclusion of implied warranties, so the above exclusion may not apply to you. The warranty provided herein gives you specific legal rights and you may also have other legal rights that vary from jurisdiction to jurisdiction. The limitations or exclusions of warranties, remedies or liability contained in this Agreement shall apply to you only to the extent such limitations or exclusions are permitted under the laws of the jurisdiction where you are located.

9\.  Indemnity

9.1 By Posit. Posit agrees to defend (at Posit’s expense) you and your Affiliates and your respective officers, directors, and employees from and against any and all claims, demands, suits, and proceedings asserted by a third party ("Third Party Claims") against any or all of them which Third Party Claims allege that the Software or your use thereof infringes the intellectual property rights of a third party and Posit shall indemnify and hold you and your indemnitees harmless from all damages finally awarded by a court of competent jurisdiction or in a binding arbitration or agreed to by Posit in settlement with respect to such Third Party Claims. Notwithstanding the foregoing, Posit shall have no obligation with respect to Third Party Claims or demands arising from (i) an allegation that does not state with specificity that the Software is the basis of the Third Party Claims; (ii) the use or combination of the Software or any part thereof with software, hardware, or other materials not developed by Posit if the Software or use thereof would not infringe without such combination; (iii) modification of the Software by a party other than Posit or its authorized contractors, if the use of unmodified Software would not constitute infringement; (iv) Third Party Materials or your Customer Applications; (v) your use of the Software in violation of this Agreement or the Documentation if the infringement would not have arisen but for such violation; (vi) your failure to install an Update that would have avoided the infringement provided that Posit made the Update available to you for free and with notice that it was provided to avoid an infringement; (vii) an allegation of infringement or misappropriation deriving from Open Source Languages or other open source software; or (viii) an allegation made against you prior to the execution of this Agreement. Further, if Posit has a reasonable belief that the Software is infringing, Posit will, at Posit’s sole cost and expense, either (a) obtain for you a license to continue using the Software, or (b) modify the Software so that it is no longer infringing without any material loss of functionality; provided that if neither option is commercially feasible within a reasonable period of time, either party may elect to terminate this Agreement upon written notice to the other party and Posit will refund all prepaid fees for the unused remainder of the Subscription Term following the termination date with respect to the Software that is the subject of the Third Party Claim. This Section 9.1 states Posit’s sole liability and your exclusive remedy for claims of infringement or misappropriation of third-party intellectual property rights.

9.2 Your Indemnity. You agree to defend (at your expense) Posit and its officers, directors, and employees from and against any and all Third Party Claims asserted against any or all of them which Third Party Claims arise out of or relate to your Customer Applications or use of the Software in violation of this Agreement, and you shall indemnify and hold Posit and its indemnitees harmless from all damages finally awarded by a court of competent jurisdiction or in a binding arbitration or agreed to by you in settlement with respect to such Third Party Claims.

9.3 Process. Each party’s obligations under this Section 9 are conditioned on the indemnified party (i) providing the indemnifying party with written notice promptly upon learning of any claim for which it seeks indemnification; provided that any failure to provide prompt notice shall not relieve the indemnifying party of its obligations except to the extent the indemnifying party was prejudiced by such failure; (ii) providing the indemnifying party sole control over the defense and settlement of the claim; provided that the indemnifying party may not settle the claim in a manner adverse to the indemnified party or which would impose liability on the indemnified party without the indemnified party’s prior written consent (which will not be unreasonably withheld or delayed); and provided, further, that the indemnified party may participate in the defense of the claim at its own cost and expense; and (iii) providing the indemnifying party with assistance in the defense and settlement of the claim at the indemnifying party’s expense.

10\. LIMITATION OF LIABILITY

10.1 Limitation of Liability. TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER PARTY SHALL BE LIABLE TO THE OTHER PARTY OR ANY THIRD PARTY FOR ANY INDIRECT, SPECIAL, INCIDENTAL, EXEMPLARY, PUNITIVE, COVER, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF BUSINESS, LOSS OF PROFITS, LOSS OF GOODWILL, LOSS OF USE, LOSS OR CORRUPTION OF DATA, INABILITY TO ACCESS OR USE EQUIPMENT, SOFTWARE OR DATA, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, BUSINESS INTERRUPTION OR THE LIKE), ARISING OUT OF THIS AGREEMENT OR THE USE OF, OR INABILITY TO USE, THE SOFTWARE, OR BASED ON ANY THEORY OF LIABILITY INCLUDING WITHOUT LIMITATION BREACH OF CONTRACT, BREACH OF WARRANTY, TORT (INCLUDING NEGLIGENCE), PRODUCT LIABILITY OR OTHERWISE, EVEN IF SUCH PARTY OR ITS REPRESENTATIVES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES AND EVEN IF A REMEDY SET FORTH HEREIN IS FOUND TO HAVE FAILED OF ITS ESSENTIAL PURPOSE. TO THE MAXIMUM EXTENT PERMITTED BY LAW, EACH PARTY’S TOTAL AGGREGATE LIABILITY UNDER THIS AGREEMENT FOR ANY CAUSE WHATSOEVER WILL NOT EXCEED THE AMOUNT PAID TO POSIT FOR THE SOFTWARE THAT CAUSED SUCH DAMAGE IN THE TWELVE MONTHS IMMEDIATELY PRIOR TO THE DATE THE CAUSE OF ACTION AROSE.

10.2 Exclusions. The foregoing limitations in Section 10.1 shall not apply to (i) gross negligence, willful misconduct, or fraud, (ii) breach of the obligations of confidentiality set forth in Section 6, (iii) violation or misappropriation by a party of the other party’s intellectual property rights, or (iv) you or your Users’ or third party hosting providers’ breach of Section 3.1 (License Grant), 3.3 (Virtualization Technology; Third Party Hosting Providers), or Section 3.4 (Restrictions; Export Laws). Further, the foregoing limitations shall not apply to a party’s indemnification obligations in Section 9 for which a party’s total aggregate liability shall be limited to (x) three times (3x) the fees paid to Posit for the Software that is the subject of the Third Party Claim in the twelve months immediately preceding the date the claim arose or (y) $100,000, whichever is greater. Further, nothing Section 10.1 shall limit your obligation to pay the applicable fee(s) for all authorized and unauthorized use of the Software.

10.3 Basis of the Bargain. The Disclaimer set forth in Section 8.2 and Limitation of Liability set forth in this Section 10 are fundamental elements of the basis of the agreement between Posit and you. Posit and its suppliers would not be able to provide the Software on an economic basis without such limitations.

11\.  Term and Termination.

11.1 Term. This Agreement shall remain in full force and effect until all Order Forms entered into hereunder have terminated or expired or until earlier terminated in accordance herewith. Each Order Form shall commence on the Subscription Start Date and shall continue for the Subscription Term set forth therein unless earlier terminated in accordance herewith. Unless either party provides the other party of written notice of termination at least 30 days prior to the end of the then current Subscription Term, the Subscription Term will be extended for an additional twelve months on each anniversary of the Subscription Start Date. Prior to the commencement of the renewal Subscription Term, you shall either submit a purchase order or make payment in full of the applicable annual license fees for such renewal Subscription Term. If you do not submit a purchase order or make payment in full on or before the commencement of the renewal Subscription Term, the Subscription Term will immediately terminate without further action by the parties.

11.2 Termination. This Agreement or an individual Order Form may be terminated (a) in the case of Posit, if you fail to pay any amount due hereunder within ten (10)business days after written demand by Posit for payment thereof, (b) by either party if the other party becomes insolvent, resolves to file bankruptcy, is adjudicated as bankrupt, or if a petition in bankruptcy is filed against the other party and such petition is not discharged within sixty (60) days of such filing, (c) by mutual written agreement of the parties, or (d) by either party if the other party materially breaches this Agreement and fails to cure such breach to such party’s reasonable satisfaction within thirty (30) days following receipt of written notice thereof.

11.3 Effect of Termination. Upon any termination of this Agreement or the affected Order Form, You and your Users shall immediately cease all use of the Software and certify in writing to Posit within thirty (30) days after termination that such Software and all copies thereof have been destroyed, purged, or returned to Posit.Termination of this Agreement or a license granted hereunder shall not limit either party from pursuing any remedies available to it, including injunctive relief, or relieve you of your obligation to pay all fees that have accrued or have become payable hereunder, without right of refund. Sections 1, 5, 6, 7, 8.2, 9, 10, 11.3 and 12 shall survive any expiration or termination of this Agreement.

12\.  General.

12.1 Governing Law; Jurisdiction. This Agreement shall be governed by the internal laws of the Commonwealth of Massachusetts, U.S.A., without giving effect to principles of conflict of laws. Each party hereby irrevocably consents to the exclusive jurisdiction and venue of the state courts sitting in the Commonwealth of Massachusetts or the federal courts in the Commonwealth of Massachusetts to resolve any disputes arising under this Agreement. In each case this Agreement shall be construed and enforced without regard to the United Nations Convention on the International Sale of Goods and without regard to the Uniform Computer Information Transactions Act. To the fullest extent permitted, each party waives the right to trial by jury in any legal proceeding arising out of or relating to this Agreement or the transactions contemplated hereby.

12.2 Assignment. Neither party may assign any of its rights or obligations hereunder, whether by operation of law or otherwise, without the prior written consent of the other party (not to be unreasonably withheld). Notwithstanding the foregoing, either party may assign this Agreement in its entirety (including all Order Forms), without consent of the other party, to its Affiliate or to its successor in a merger, acquisition, corporate reorganization, or sale of all or substantially all of its assets, stock, or business to which this Agreement relates. Subject to the foregoing, this Agreement shall bind and inure to the benefit of the parties, their respective successors and permitted assigns.

12.3 Entire Agreement. This Agreement (including all Order Form(s) entered into by the parties, the Software License Descriptions and all documents linked to herein and therein) contain the complete agreement between the parties with respect to the subject matter hereof, and supersede all prior and contemporaneous agreements and understandings whether oral or written. You agree that any varying or additional terms contained in any purchase order or other written notification or document issued by you in relation to the Software licensed hereunder shall be of no effect and all such terms or conditions shall be null and void. You acknowledge and agree that your agreement hereunder is not contingent upon the delivery of any future functionality or features not specified herein or in an Order Form or dependent upon any oral or written, public or private comments made by Posit with respect to future functionality or features for the Software. In the event of any conflict between this Agreement, the Software License Descriptions, any Order Form or any document linked to herein or therein, this Agreement shall govern; provided, however, that notwithstanding the foregoing and for the avoidance of doubt: (i) the license limitations and entitlements set forth in an Order Form and the Software License Descriptions shall supplement this Agreement and shall supersede any conflicting terms for purposes of determining the license limitations and entitlements applicable to your and your Users use of the Software and (ii) the Support Program shall supplement this Agreement and shall supersede any conflicting terms for purposes of defining your Support Services entitlements.

12.4 Waiver. The failure or delay of a party to exercise any of its rights under this Agreement or upon any breach of this Agreement shall not be deemed a waiver of those rights or of the breach. No Reseller or Posit dealer or agent is authorized to make any amendment to this Agreement. The parties may amend this Agreement by mutual written agreement executed by duly authorized representatives of the parties. The parties agree that there are no third party beneficiaries to this Agreement. Neither party shall be deemed to be an employee, agent, or other legal representative of the other party for any purpose whatsoever, or have the right or authority to assume or otherwise create any obligation or responsibility, express or implied, on behalf of the other party or to bind the other party in any manner whatsoever.

12.5 Severability. If any provision of this Agreement shall be held by a court of competent jurisdiction to be contrary to law that provision will be enforced to the maximum extent permissible and the remaining provisions of this Agreement will remain in full force and effect. A waiver by either party of any term or condition of this Agreement or any breach thereof, in any one instance, shall not waive such term or condition or any subsequent breach thereof.

12.6 Notices. Notices must be in English, in writing, and will be deemed given upon receipt, after being sent using a method that provides for positive confirmation of delivery to the physical address or email address set forth in the Order Form, including without limitation through automated receipt or by electronic log.

12.7 U.S. Government Users. The Software and its Documentation are "Commercial items," "Commercial computer software" and "Computer software documentation" as defined by the Federal Acquisition Regulations ("FAR") and Defense Federal Acquisition Regulations Supplement ("DFARS"). Pursuant to FAR 12.211, FAR 12.212, DFARS, 227.7202-1 through 227.7202-4, and their successors, the U.S. Government acquires the Software and its documentation subject to the terms of this Agreement.

12.8 Counterparts; Electronic Signatures; Construction. This Agreement and any Order Form may be executed in counterparts, which taken together shall form one legal instrument. A manually or electronically signed copy of this Agreement or any Order Form delivered by facsimile, e-mail or other means of electronic transmission shall be deemed to have the same legal effect as delivery of an original signed copy of the Agreement or the Order Form. The titles of the sections of this Agreement are for convenience of reference only and are not to be considered in construing this Agreement. Unless the context of this Agreement clearly requires otherwise: (i) references to the plural include the singular, the singular the plural, and the part the whole, (ii) "or" has the inclusive meaning frequently identified with the phrase "and/or," (iii) "including" has the inclusive meaning frequently identified with the phrase "including but not limited to" or "including without limitation," and (iv) references to "hereunder," "herein" or "hereof" relate to this Agreement as a whole. Any reference in this Agreement to any statute, rule, regulation or agreement, including this Agreement, shall be deemed to include such statute, rule, regulation or agreement as it may be modified, varied, amended or supplemented from time to time. The parties agree that this Agreement shall be fairly interpreted in accordance with its terms without any strict construction in favor of or against either party and that ambiguities shall not be interpreted against the drafting party.  
