---
title: "Simplified Interface"
date-meta: 2022-12-12
---

RStudio has many features and options to help you work the way that best suits your needs.

Some features discussed are specific to RStudio Desktop or RStudio Server: if RStudio is loaded in your web browser, then you are using RStudio Server; otherwise, you have RStudio Desktop.

For the purposes of this discussion, "RStudio Server" encompasses both RStudio Server Open Source and Posit Workbench (previously RStudio Workbench or RStudio Server Pro).

Email accessibility-related questions and comments to [accessibility\@posit.co](mailto:accessibility@posit.co).

## Zoom level

The size of text and other RStudio user-interface elements can be adjusted using the Zoom feature.

For RStudio Desktop, use the "Zoom In", "Zoom Out", or "Actual Size" commands on the View menu.

RStudio Server relies on your web browser's zooming features. Consult web browser help for more details.

-   [Google Chrome Zoom Documentation](https://support.google.com/chrome/answer/96810)

-   [Firefox Zoom Documentation](https://support.mozilla.org/en-US/kb/font-size-and-zoom-increase-size-of-web-pages)

-   [Safari on MacOS Zoom Documentation](https://support.apple.com/guide/safari/zoom-in-on-webpages-ibrw1068)

-   [Microsoft Edge Accessibility Features](https://support.microsoft.com/en-us/help/4000734/microsoft-edge-accessibility-features)

## Look and feel

You can change the look and feel of the RStudio user interface by choosing from three RStudio themes, and many editor themes. Use the Tools / Global Options / Appearance settings pane to experiment and find the look that works best for you.

Pebble-Safe is a colorblind-friendly theme created by Desi Quintans. It can be downloaded and installed from the [Pebble-Safe GitHub page](https://github.com/DesiQuintans/Pebble-safe).

More details on RStudio's theming features are in the [Using RStudio Themes article](../ui/appearance.html).

## Keyboard

Many RStudio features can be operated entirely with the keyboard. Using the keyboard can be an efficient way to get things done whether you are a keyboard-only user, or just prefer to reduce how often you have to reach for the mouse or touchpad.

### Keyboard shortcuts

RStudio has many keyboard shortcuts to trigger actions, move between areas of the user interface, and so on. Many of these can be discovered by browsing the main menu, hovering the mouse cursor over toolbar buttons, and viewing keyboard shortcut reference from the Help / Keyboard Shortcut Help menu.

Keyboard shortcuts may be customized using the Tools / Modify Keyboard Shortcuts menu. For more on this, see the [Customizing Keyboard Shortcuts article.](../productivity/custom-shortcuts.html)

Moving between RStudio's various tabs such as the Console and the Source Editor can be achieved using shortcuts such as Ctrl+1 to move to the editor tab, and Ctrl+2 to move to the Console. Many more such commands are found under the View menu.

The keyboard shortcuts available for manipulating text in the Source Editor will vary depending which editor keybindings are selected in Global Options / Code / Keybindings.

### Main menu

In RStudio Desktop, the main menu is operated via keyboard the same as the main menu in other applications on your operating system. For example, on Microsoft Windows, Alt+F will open the file menu.

RStudio Server, however, has slightly different shortcuts for activating its main menu items. As a starting point, on Windows and Linux, use Alt+Shift+F to activate the File menu, or Alt+Shift+H for the Help menu. On macOS, use Control+Option+F and Control+Option+H. Once a menu is open, use the left and right arrow keys to move between menus, and up and down arrows to navigate within the current menu, just like on RStudio Desktop.

A full list of Main Menu shortcuts is available in the Help / Keyboard Shortcuts reference, under the title "Main Menu (Server)".

If you wish to customize these shortcuts, they are named "Show File Menu", "Show Help Menu", and so on.

### Navigating dialog boxes

When a dialog box such as Global Options is opened, you can move around it using a combination of arrow keys and Tab and Shift+Tab. Generally the spacebar and/or Enter key can be used to "click" a button, just like other desktop applications.

## Reduced motion

Some aspects of RStudio's user interface use subtle motion or animations. There is an option to eliminate these via the **Tools** > **Global Options** > **Accessibility** > **Reduce User Interface Animations** checkbox.

## Simplify layout

RStudio's main window is quite visually complex. There is a menubar at the top, a toolbar, and up to four quadrants each potentially containing many different types of tabs which in turn contain RStudio's many features! Additionally, some features will open entirely new windows.

This can be overwhelming when first learning to use RStudio, and potentially distracting even if you are experienced with the product. Here are some things you can try.

### Hiding toolbar

If you are not going to use it, the main toolbar can be hidden via View / Hide Toolbar, and redisplayed with View / Show Toolbar.

### Zooming a region

A strategy you can employ is to "zoom" the region you are currently using. For example, View / Panes / Zoom Console will cause the Console to fill the entire RStudio Window. Note this is a different meaning of "Zoom" than was mentioned earlier in this document where it referred to changing the overall size of all elements in the window.

The keyboard shortcut for zooming a pane is often the same as for moving to the pane, plus the Shift key. So, Ctrl+Shift+2 will zoom the Console; use the same sequence to un-zoom the Console and restore the prior layout.

For mouse users, there are also widgets in the upper-right corner of each quadrant which can be used to maximize and minimize that particular quadrant.

Another strategy to employ is zooming a single column of the user interface. For example, in the default layout the Source Editor is in the top-left (assuming you have some files open) and the Console is below it. If you then use View / Panes / Zoom Left Column, those two panes will expand to fill the entire main window.

### Customizing pane layout

Another tool you can experiment with is customizing which quadrants contain certain tabs, as well as where each quadrant is displayed relative to the others. This is configured via View / Panes / Pane Layout dialog (also available within the Global Options dialog under Pane Layout).

The Source and Console quadrants are currently fixed in terms of what tab-types may appear in them. You can, however, configure the other two quadrants to hold specific tabs. The only limitation is that a quadrant must contain at least one tab, so it is not possible to completely eliminate one quadrant. Still, you can move the tabs you don't routinely use into one quadrant then minimize that quadrant, giving what is essentially a 3 panel layout.

## Accessibility options

The Global Options dialog contains a section dedicated to accessibility options. Options for reducing motion in the user interface, and for toggling the Tab key to always move focus, were mentioned earlier in this article. The rest of the options are specific to screen reader usage, and are discussed in the [section on RStudio Screen Reader support](screen-reader.html).

## Resources

Here are some additional resources related to RStudio accessibility, and accessibility of the R ecosystem.

-   [RStudio Blog: 1.3 Preview Accessibility](https://posit.co/blog/rstudio-1-3-preview-accessibility/)

-   [BlindRUG - Blind R Users Group](http://nfbnet.org/mailman/listinfo/blindrug_nfbnet.org)

-   [Statistical Software and Blind Users](https://r-resources.massey.ac.nz/StatSoftware/)
