This folder contains the RStudio UI code, written using the Google Web Toolkit (GWT).

RStudio uses a custom fork of GWT, with the pre-built custom JARS installed as a dependency.

The source code for this fork can be found at https://github.com/rstudio/gwt. Only read this
if working on something requiring deep knowledge of GWT internals that can't be figured out
from existing usage and the documentation.
