---
name: Bug report
about: Report a bug in RStudio.
title: ''
labels: ['bug']
assignees: ''

---

<!--

IMPORTANT: Please provide a reproducible example! This can be a snippet of code, or set of actions, that we can use to reproduce the issue you are encountering. We are typically unable to fix issues that we cannot reproduce locally -- if we can reproduce the issue you are encountering on our own machines, then we are much more likely to be able to fix the issue.

This issue tracker is for bugs and feature requests in the RStudio IDE. If you're having trouble with R itself or an R package, see https://www.r-project.org/help.html, and if you want to ask a question rather than report a bug, go to https://forum.posit.co/. Finally, if you use RStudio Server Pro, get in touch with Posit Support at support@posit.co.

-->

### System details

    RStudio Edition : <!-- Desktop or Server -->
    RStudio Version : 
    OS Version      : 
    R Version       : 

### Steps to reproduce the problem

### Describe the problem in detail

### Describe the behavior you expected


<!-- 
Please keep the below portion in your issue, and check `[x]` the applicable boxes.
-->

- [ ] I have read the guide for [submitting good bug reports](https://github.com/rstudio/rstudio/wiki/Writing-Good-Bug-Reports).
- [ ] I have installed the latest version of RStudio, and confirmed that the issue still persists.
- [ ] If I am reporting an RStudio crash, I have included a [diagnostics report](https://support.posit.co/hc/en-us/articles/200321257-Running-a-Diagnostics-Report).
- [ ] I have done my best to include a minimal, self-contained set of instructions for consistently reproducing the issue.
