---
title: "RStudio IDE User Guide"
subtitle: "RStudio is an integrated development environment (IDE) designed to support multiple languages, including both R and Python."
date: today
description: "Revision {{< var revision >}}"
date-format: iso
---

RStudio is available in open source and commercial editions and runs on the desktop (Windows 11, macOS 14+, and Linux) or in a browser connected to RStudio Server or Posit Workbench. Development of the open source and commercial RStudio IDE variants is supported by Posit Software, PBC (formerly named RStudio, PBC). Please review our [Blog post/FAQ](https://pos.it/rebrand-faq/) for additional information about the rebrand.

RStudio includes a console, syntax-highlighting editor that supports direct code execution, and a variety of robust tools for plotting, viewing history, debugging and managing your workspace. 

Please visit the individual product pages for additional information about the open source [RStudio IDE](https://posit.co/products/open-source/rstudio/) or RStudio within [Posit Workbench](https://posit.co/products/enterprise/workbench/). 

Some of the functionality that RStudio provides is:

-   Access RStudio locally.  
-   Syntax highlighting, code completion, and smart indentation.  
-   Execute R code directly from the source editor.  
-   Quickly jump to function definitions.  
-   View content changes in real-time with the Visual Markdown Editor for RMarkdown or Quarto documents.  
-   Easily manage multiple working directories using Projects.  
-   Integrated R help and documentation.  
-   View Python data, publish and render in Python and share objects with R.  
-   Interactive debugger to diagnose and fix errors.  
-   Extensive package development tools.  
-   Easily publish apps and reports.  

## About this guide

The following documentation helps a user understand the core workflows in RStudio. It provides information for navigating the user interface and appearance, managing R and R packages, working with Python, authoring code or text editing, debugging and diagnostics, using the Visual Editor for Quarto and RMarkdown, reading in data from local and remote sources, and integrations with version control systems and the terminal.

### Intended audience

This guide is intended for RStudio IDE users (e.g., data scientists).

:::{.callout-note}
This guide is intended for RStudio IDE users.

Posit Workbench users (e.g., data scientists) can reference the [Posit Workbench User Guide](https://docs.posit.co/ide/server-pro/user/).

Posit Workbench administrators can reference the [Posit Workbench Administrator Guide](https://docs.posit.co/ide/server-pro/).

:::

## Commercial Edition

Posit Workbench has several features that differentiate it from the open-source [RStudio desktop](https://posit.co/products/open-source/rstudio/) and [RStudio Server](https://posit.co/products/open-source/rstudio-server/), which you can read about on the [Posit Workbench comparison page](https://posit.co/products/enterprise/workbench/). The main advantages of using Posit Workbench are:

- Enhanced security and authentication, including Single Sign On (SSO).  
- Run multiple-concurrent sessions.  
- Use multiple different versions of R and Python.  
- Support for multiple IDEs such as VSCode and JupyterLab, in addition to RStudio.  
- Collaborate on projects.  
- Easily connect to shared resources such as databases or mounted file systems.  
- Launch jobs into remote or local clusters using Workbench Jobs.  
- The ability to fine-tune the performance of the user experience.  
- The ability to monitor the performance of sessions/users.  
- A commercial license to remove the restrictions of the AGPL license.  

## Install Links

### Open Source

- [RStudio Desktop download](#rstudio-ide-oss-downloads) - available for Windows/Mac/Linux
- [RStudio Server download](#rstudio-server-oss-downloads) - available for Linux

### Commercial

- [Posit Workbench](https://docs.posit.co/rsw/installation/) - administration guide for installation and configuration
- [RStudio Desktop Pro](https://docs.posit.co/ide/desktop-pro/getting_started/installation.html#direct-downloads) - available for Windows/Mac/Linux

### Direct Downloads (Open Source)

#### RStudio IDE {#rstudio-ide-oss-downloads}

Direct download links for RStudio Desktop {{< var version >}}, the version this documentation describes:

{{< include ../../_snippets/install-desktop-oss.qmd >}}

#### RStudio Server {#rstudio-server-oss-downloads}

Direct download links for RStudio Server {{< var version >}}, the version this documentation describes:

{{< include ../../_snippets/install-server-oss.qmd >}}

### Direct Downloads (Commercial)

Direct download links for RStudio Desktop Pro {{< var version >}}, the version this documentation describes:

{{< include ../../_snippets/install-desktop-pro.qmd >}}

::: {.callout-warning }

### Daily Builds

[RStudio Daily builds](https://dailies.rstudio.com/) are intended for testing purposes, and are not recommended for general use. For stable builds, please use one of the above links for Open Source or Commercial downloads.

:::

