---
title: "Untitled"
output: html_document
---

This is an R Markdown document. Markdown is a simple formatting syntax for authoring HTML, PDF, and MS Word documents. For more details on using R Markdown see <http://rmarkdown.rstudio.com>.

When you click the **Knit** button a document will be generated that includes both content as well as the output of any embedded R code chunks within the document. You can embed an R code chunk like this:

# A Markdown Header

Another Markdown Header
=======================

## A Sub-Header

## A Second Sub-Header

```{r first-chunk}
```

```{r second-chunk}
foo <- function(x) {
  bar <- function(y) {
    bar(1)
  }
}
```

```{r third-chunk}

```
